' ========================================================================================
' Recording Metafiles
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example creates a Metafile. The code uses the Metafile to record a
' sequence of graphics commands and then saves the recorded commands in a file named
' SampleMetafile.emf. Note that the function that creates the metafile
' (GdipRecordMetafileFileName) receives a device context handle, and we get a Graphics
' handle calling the GdipGetImageGraphicsContext function passing the metafile handle.
' The recording stops (and the recorded commands are saved to the file) when the Graphics
' handle is deleted with GdipDeleteGraphics. To display the metafile, a new Graphics
' handle is retrieved calling the GdipCreateFromHDC function and passing the Graphics and
' Metafile handles to the GdipDrawImage function. Note that the code uses the same
' Metafile handle to record and to display (play back) the metafile.
' A metafile contains its own graphics state, which is defined by the Graphics object used
' to record the metafile. Any properties of the Graphics object (clip region, world
' transformation, smoothing mode, and the like) that you set while recording the metafile
' will be stored in the metafile. When you display the metafile, the drawing will be done
' according to those stored properties.
' ========================================================================================
SUB GDIP_RecordMetafile (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pMetafile AS DWORD
   LOCAL pGraphics AS DWORD
   LOCAL pPlaybackGraphics AS DWORD
   LOCAL pGreenPen AS DWORD
   LOCAL pSolidBrush AS DWORD
   LOCAL pFontFamily AS DWORD
   LOCAL pFont AS DWORD
   LOCAL strFileName AS STRING
   LOCAL strDesc AS STRING
   LOCAL strFontName AS STRING
   LOCAL strText AS STRING
   LOCAL rcf AS RECTF

   ' // Create a Metafile object for recording
   strFileName = UCODE$("SampleMetafile.emf")
   strDesc = UCODE$("")
   hStatus = GdipRecordMetafileFileName(STRPTR(strFileName), hdc, _
             %EmfTypeEmfPlusDual, BYVAL %NULL, %MetafileFrameUnitGdi, _
             STRPTR(strDesc), pMetafile)

   IF pMetaFile THEN
      ' // Create a Graphics object that is associated with the Metafile object
      hStatus = GdipGetImageGraphicsContext(pMetafile, pGraphics)
      IF pGraphics THEN
         ' // Create a pen and a solid brush
         hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 255, 0), 1, %UnitPixel, pGreenPen)
         hStatus = GdipCreateSolidFill(GDIP_ARGB(255, 0, 0, 255), pSolidBrush)
         ' // Add a rectangle and an ellipse to the metafile
         hStatus = GdipDrawRectangle(pGraphics, pGreenPen, 50, 10, 25, 75)
         hStatus = GdipDrawEllipse(pGraphics, pGreenPen, 100, 10, 25, 75)
         ' // Add an ellipse (drawn with antialiasing) to the metafile.
         hStatus = GdipSetSmoothingMode(pGraphics, %SmoothingModeHighQuality)
         hStatus = GdipDrawEllipse(pGraphics, pGreenPen, 150, 10, 25, 75)
         ' // Add some text (drawn with antialiasing) to the metafile.
         strFontName = UCODE$("Arial")
         hStatus = GdipCreateFontFamilyFromName(STRPTR(strFontName), %NULL, pFontFamily)
         IF pFontFamily THEN
            hStatus = GdipCreateFont(pFontFamily, 24, %FontStyleRegular, %UnitPixel, pFont)
            GdipDeleteFontFamily(pFontFamily)
         END IF
         hStatus = GdipSetTextRenderingHint(pGraphics, %TextRenderingHintAntiAlias)
         hStatus = GdipRotateWorldTransform(pGraphics, 30.0!, %MatrixOrderPrepend)
         strText = UCODE$("Smooth Text")
         rcf.x = 50.0! : rcf.y = 50.0!
         hStatus = GdipDrawString(pGraphics, STRPTR(strText), LEN(strText) \ 2, pFont, rcf, %NULL, pSolidBrush)
         ' // Stop recording by deleting the Graphics object
         GdipDeleteGraphics(pGraphics)
         ' // Free resources
         IF pFont THEN GdipDeleteFont(pFont)
         IF pSolidBrush THEN GdipDeleteBrush(pSolidBrush)
         IF pGreenPen THEN GdipDeletePen(pGreenPen)
      END IF
   END IF

   ' // Create a Graphics object that is associated with the device context
   hStatus = GdipCreateFromHDC(hdc, pPlaybackGraphics)
   IF pPlaybackGraphics THEN
      ' // Display the metafile
      hStatus = GdipDrawImage(pPlaybackGraphics, pMetafile, 200, 100)
      ' // Delete the Graphics object
      GdipDeleteGraphics(pPlaybackGraphics)
   END IF

   ' // Dispose the Metafile object
   IF pMetafile THEN GdipDisposeImage(pMetafile)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hWndMain AS DWORD
   LOCAL hCtl AS DWORD
   LOCAL hFont AS DWORD
   LOCAL wcex AS WndClassEx
   LOCAL szClassName AS ASCIIZ * 80
   LOCAL rc AS RECT
   LOCAL szCaption AS ASCIIZ * 255
   LOCAL nLeft AS LONG
   LOCAL nTop AS LONG
   LOCAL nWidth AS LONG
   LOCAL nHeight AS LONG
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   hFont = GetStockObject(%ANSI_VAR_FONT)

   ' Register the window class
   szClassName        = "MyClassName"
   wcex.cbSize        = SIZEOF(wcex)
   wcex.style         = %CS_HREDRAW OR %CS_VREDRAW
   wcex.lpfnWndProc   = CODEPTR(WndProc)
   wcex.cbClsExtra    = 0
   wcex.cbWndExtra    = 0
   wcex.hInstance     = hInstance
   wcex.hCursor       = LoadCursor (%NULL, BYVAL %IDC_ARROW)
   wcex.hbrBackground = GetStockObject(%WHITE_BRUSH)
   wcex.lpszMenuName  = %NULL
   wcex.lpszClassName = VARPTR(szClassName)
   wcex.hIcon         = LoadIcon (%NULL, BYVAL %IDI_APPLICATION) ' Sample, if resource icon: LoadIcon(hInst, "APPICON")
   wcex.hIconSm       = LoadIcon (%NULL, BYVAL %IDI_APPLICATION) ' Remember to set small icon too..
   RegisterClassEx wcex

   ' Window caption
   szCaption = "Recording Metafile Demo"

   ' Retrieve the nSize of the working area
   SystemParametersInfo %SPI_GETWORKAREA, 0, BYVAL VARPTR(rc), 0

   ' Calculate the position and nSize of the window
   nWidth  = (((rc.nRight - rc.nLeft)) + 2) * 0.70   ' 70% of the client screen width
   nHeight = (((rc.nBottom - rc.nTop)) + 2) * 0.70   ' 70% of the client screen height
   nLeft   = ((rc.nRight - rc.nLeft) \ 2) - nWidth \ 2
   nTop    = ((rc.nBottom - rc.nTop) \ 2) - (nHeight \ 2)

   ' Create a window using the registered class
   hWndMain = CreateWindowEx(%WS_EX_CONTROLPARENT, _           ' extended style
                             szClassName, _                    ' window class name
                             szCaption, _                      ' window caption
                             %WS_OVERLAPPEDWINDOW OR _
                             %WS_CLIPCHILDREN, _               ' window style
                             nLeft, _                          ' initial x position
                             nTop, _                           ' initial y position
                             nWidth, _                         ' initial x nSize
                             nHeight, _                        ' initial y nSize
                             %NULL, _                          ' parent window handle
                             0, _                              ' window menu handle
                             hInstance, _                      ' program instance handle
                             BYVAL %NULL)                      ' creation parameters

   ' Show the window
   ShowWindow hWndMain, nCmdShow
   UpdateWindow hWndMain

   ' Message handler loop
   LOCAL Msg AS tagMsg
   WHILE GetMessage(Msg, %NULL, 0, 0)
      IF ISFALSE IsDialogMessage(hWndMain, Msg) THEN
         TranslateMessage Msg
         DispatchMessage Msg
      END IF
   WEND

   ' Shutdown GDI+
   GdiplusShutdown token

   FUNCTION = msg.wParam

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Window procedure
' ========================================================================================
FUNCTION WndProc (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE wMsg

      CASE %WM_COMMAND
         ' -------------------------------------------------------
         ' Messages from controls and menu items are handled here.
         ' -------------------------------------------------------
         SELECT CASE LOWRD(wParam)

            CASE %IDCANCEL
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  SendMessage hWnd, %WM_CLOSE, 0, 0
                  FUNCTION = 0
                  EXIT FUNCTION
               END IF

         END SELECT

      CASE %WM_SYSCOMMAND
         ' Capture this message and send a WM_CLOSE message
         IF (wParam AND &HFFF0) = %SC_CLOSE THEN
            SendMessage hWnd, %WM_CLOSE, 0, 0
            EXIT FUNCTION
         END IF

      CASE %WM_PAINT
         hDC = BeginPaint(hWnd, ps)
         GDIP_RecordMetafile hDC
         EndPaint(hWnd, ps)

      CASE %WM_DESTROY
         PostQuitMessage 0    ' This function closes the main window
         FUNCTION = 0         ' by sending zero to the main message loop
         EXIT FUNCTION

   END SELECT

   FUNCTION = DefWindowProc(hWnd, wMsg, wParam, lParam)

END FUNCTION
' ========================================================================================
