' ========================================================================================
' Recording Metafiles
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

%IDC_GRAPHIC  = 101

' ========================================================================================
' The following example creates a Metafile. The code uses the Metafile to record a
' sequence of graphics commands and then saves the recorded commands in a file named
' SampleMetafile.emf. Note that the function that creates the metafile
' (GdipRecordMetafileFileName) receives a device context handle, and we get a Graphics
' handle calling the GdipGetImageGraphicsContext function passing the metafile handle.
' The recording stops (and the recorded commands are saved to the file) when the Graphics
' handle is deleted with GdipDeleteGraphics. To display the metafile, a new Graphics
' handle is retrieved calling the GdipCreateFromHDC function and passing the Graphics and
' Metafile handles to the GdipDrawImage function. Note that the code uses the same
' Metafile handle to record and to display (play back) the metafile.
' A metafile contains its own graphics state, which is defined by the Graphics object used
' to record the metafile. Any properties of the Graphics object (clip region, world
' transformation, smoothing mode, and the like) that you set while recording the metafile
' will be stored in the metafile. When you display the metafile, the drawing will be done
' according to those stored properties.
' ========================================================================================
SUB GDIP_RecordMetafile (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pMetafile AS DWORD
   LOCAL pGraphics AS DWORD
   LOCAL pPlaybackGraphics AS DWORD
   LOCAL pGreenPen AS DWORD
   LOCAL pSolidBrush AS DWORD
   LOCAL pFontFamily AS DWORD
   LOCAL pFont AS DWORD
   LOCAL strFileName AS STRING
   LOCAL strDesc AS STRING
   LOCAL strFontName AS STRING
   LOCAL strText AS STRING
   LOCAL rcf AS RECTF

   ' // Create a Metafile object for recording
   strFileName = UCODE$("SampleMetafile.emf")
   strDesc = UCODE$("")
   hStatus = GdipRecordMetafileFileName(STRPTR(strFileName), hdc, _
             %EmfTypeEmfPlusDual, BYVAL %NULL, %MetafileFrameUnitGdi, _
             STRPTR(strDesc), pMetafile)

   IF pMetaFile THEN
      ' // Create a Graphics object that is associated with the Metafile object
      hStatus = GdipGetImageGraphicsContext(pMetafile, pGraphics)
      IF pGraphics THEN
         ' // Create a pen and a solid brush
         hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 255, 0), 1, %UnitPixel, pGreenPen)
         hStatus = GdipCreateSolidFill(GDIP_ARGB(255, 0, 0, 255), pSolidBrush)
         ' // Add a rectangle and an ellipse to the metafile
         hStatus = GdipDrawRectangle(pGraphics, pGreenPen, 50, 10, 25, 75)
         hStatus = GdipDrawEllipse(pGraphics, pGreenPen, 100, 10, 25, 75)
         ' // Add an ellipse (drawn with antialiasing) to the metafile.
         hStatus = GdipSetSmoothingMode(pGraphics, %SmoothingModeHighQuality)
         hStatus = GdipDrawEllipse(pGraphics, pGreenPen, 150, 10, 25, 75)
         ' // Add some text (drawn with antialiasing) to the metafile.
         strFontName = UCODE$("Arial")
         hStatus = GdipCreateFontFamilyFromName(STRPTR(strFontName), %NULL, pFontFamily)
         IF pFontFamily THEN
            hStatus = GdipCreateFont(pFontFamily, 24, %FontStyleRegular, %UnitPixel, pFont)
            GdipDeleteFontFamily(pFontFamily)
         END IF
         hStatus = GdipSetTextRenderingHint(pGraphics, %TextRenderingHintAntiAlias)
         hStatus = GdipRotateWorldTransform(pGraphics, 30.0!, %MatrixOrderPrepend)
         strText = UCODE$("Smooth Text")
         rcf.x = 50.0! : rcf.y = 50.0!
         hStatus = GdipDrawString(pGraphics, STRPTR(strText), LEN(strText) \ 2, pFont, rcf, %NULL, pSolidBrush)
         ' // Stop recording by deleting the Graphics object
         GdipDeleteGraphics(pGraphics)
         ' // Free resources
         IF pFont THEN GdipDeleteFont(pFont)
         IF pSolidBrush THEN GdipDeleteBrush(pSolidBrush)
         IF pGreenPen THEN GdipDeletePen(pGreenPen)
      END IF
   END IF

   ' // Create a Graphics object that is associated with the device context
   hStatus = GdipCreateFromHDC(hdc, pPlaybackGraphics)
   IF pPlaybackGraphics THEN
      ' // Display the metafile
      hStatus = GdipDrawImage(pPlaybackGraphics, pMetafile, 200, 100)
      ' // Delete the Graphics object
      GdipDeleteGraphics(pPlaybackGraphics)
   END IF

   ' // Dispose the Metafile object
   IF pMetafile THEN GdipDisposeImage(pMetafile)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hDlg AS DWORD
   LOCAL hdc AS DWORD
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   ' Create a new dialog
   DIALOG NEW PIXELS, 0, "Recording Metafile Demo", , , 550, 360, %WS_SYSMENU TO hDlg

   ' Add a graphic control
   CONTROL ADD GRAPHIC, hDlg, %IDC_GRAPHIC,"", 0, 0, 550, 360
   ' Select the drawing target
   GRAPHIC ATTACH hDlg, %IDC_GRAPHIC
   ' Set the foreground and background color
   GRAPHIC COLOR %BLACK, %WHITE
   ' Clear the entire selected graphic window
   GRAPHIC CLEAR
   ' Retrieve the handle of the device context
   GRAPHIC GET DC TO hdc
   ' Draw the graphics
   GDIP_RecordMetafile hdc

   DIALOG SHOW MODAL hDlg, CALL DlgProc

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Dialog procedure
' ========================================================================================
CALLBACK FUNCTION DlgProc() AS LONG

   SELECT CASE CBMSG

      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDCANCEL
               IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL, 0
         END SELECT

   END SELECT

END FUNCTION
' ========================================================================================
